@@ -1,4 +1,5 @@
-from ezcode.array.utils import swap
+from ezcode.array.utils import swap, reverse
+from ezcode.array.search import binary_search_subarray_exclusive_boundery
 
 
 def permutation_size(total_size: int, selection_size: int) -> int:
@@ -39,6 +40,24 @@ def permutations(selection_size: int, items: list) -> list:
     return result
 
 
+def next_lexicographic_permutation(items: list, copy=True) -> list:
+    if copy:
+        items = items.copy()
+    begin = -1
+    for i in range(len(items) - 2, -1, -1):
+        if items[i] < items[i + 1]:
+            begin = i
+            break
+    if begin < 0:
+        reverse(items)
+    else:
+        first_greater_from_end = binary_search_subarray_exclusive_boundery(
+            items, begin=begin + 1, end=len(items) - 1, target=items[begin], is_ascending=False, is_smaller=False)
+        swap(items, begin, first_greater_from_end)
+        reverse(items, begin + 1, len(items) - 1)
+    return items
+
+
 def all_items_permutations(items: list) -> list:
     """ Equals to permutations(len(items, items)) """
     def _permutations_with_all_items(items: list, current_index, result: list):
@@ -135,3 +154,36 @@ def all_subsets(items: list, has_duplicate=True):
     result.sort(key=lambda x: len(x))  # nice to have
     return result
 
+
+def partitions(string: str) -> list:
+    """
+        '1234' -> [
+            ['1', '234'],
+            ['1', '2', '34'],
+            ['1', '2', '3', '4'],
+            ['1', '23', '4'],
+            ['12', '34'],
+            ['12', '3', '4'],
+            ['123', '4']
+        ]
+    """
+    return list(partition_generator(string))
+
+
+def partition_generator(string: str):
+    yield [string]
+    for i in range(1, len(string)):
+        for parts in partition_generator(string[i:]):
+            yield [string[0:i]] + parts
+
+
+
+
+
+
+
+
+
+
+
+
